/////////////////////////////////////////////////////////////////////////////
//
// FPGA Universal HID Bus SpartanII JTAG Controll for TimePilot
//
// Version : 1.00
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// License:
//   This source is under the GPL license.
//
// Histry:
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// pin assign
//
// HID[7:0] : Don't care
//
// jtagmem(JTAG-MEMORY CONTROLLER) Address assign
//
// JA[ 7: 0] : Player1 ST,COIN,B2,B1,DN,RT,LT,UP
// JA[15: 8] : Player2 ST,COIN,B2,B1,DN,RT,LT,UP
// JA[15: 8] : Player2 ST,COIN,B2,B1,DN,RT,LT,UP
// JA[19:18] : Player1 B6,B5,B4,B3
// JA[23:20] : Player2 B6,B5,B4,B3
// JA[24]    : Credit Service
// JA[25]    : TEST MODE
// JA[30]    : Reset (jtagmem BusMaster)
//

module hid_jmem(
  CLK,
// Universal HID Bus
  HID_I,HID_O,HID_T,
// Internal Controll
  UP,DN,LT,RT,B1,B2,
  COIN,START,SERVICE,RESET_N,
// External Controll
  LED
);

input CLK;
input [7:0] HID_I;
output [7:0] HID_O;
output [7:0] HID_T;

output [1:0] UP,DN,LT,RT,B1,B2;
output [1:0] COIN,START,SERVICE,RESET_N;

input [1:0] LED;

/////////////////////////////////////////////////////////////////////////////
// JTAG External Access
/////////////////////////////////////////////////////////////////////////////
wire [30:0] JA;
jtagmem jtagmem(
  .JA(JA),
  .JD_I(),.JD_O(),.JD_E(),
  .JCS(),.JOE(),.JWE()
);

//                  Player2  , Player1
assign UP[1:0]    = { JA[ 8] , JA[ 0] };
assign LT[1:0]    = { JA[ 9] , JA[ 1] };
assign RT[1:0]    = { JA[10] , JA[ 2] };
assign DN[1:0]    = { JA[11] , JA[ 3] };
assign B1[1:0]    = { JA[12] , JA[ 4] };
assign B2[1:0]    = { JA[13] , JA[ 5] };
assign COIN[1:0]  = { JA[14] , JA[ 6] };
assign START[1:0] = { JA[15] , JA[ 7] };

assign SERVICE = 1'b1;
assign RESET_N = JA[30];

/////////////////////////////////////////////////////////////////////////////
// U HID Bus
/////////////////////////////////////////////////////////////////////////////
assign HID_T = 8'b11111111;
assign HID_O = HID_I;

endmodule
