//
// Z80 IP interface jtameme access
//
// Version : 1.00
//
// Copyright(c) 2002,2003 Tatsuyuki Satoh , All rights reserved
//

module z80_jmem(
	RESET_n,CLK_n,WAIT_n,INT_n,NMI_n,BUSRQ_n,
	M1_n,MREQ_n,IORQ_n,RD_n,WR_n,RFSH_n,HALT_n,BUSAK_n,
	A,DI,DO,DOE);

input RESET_n,CLK_n,WAIT_n,INT_n,NMI_n,BUSRQ_n;
output M1_n,MREQ_n,IORQ_n,RD_n,WR_n,RFSH_n,HALT_n,BUSAK_n;
output [15:0] A;
input [7:0] DI;
output [7:0] DO;
output DOE;

/////////////////////////////////////////////////////////////////////////////
// JTAG MEM I/F
/////////////////////////////////////////////////////////////////////////////
wire [30:0] JA;
wire [7:0] JDI,JDO;
wire JDE,JCS,JOE,JWE;
jtagmem jtagmem(
  .JA(JA),
  .JD_I(JDI),.JD_O(JDO),.JD_E(JDE),
  .JCS(JCS),.JOE(JOE),.JWE(JWE)
);

reg [15:0] za_r;
reg [7:0] zdi_r;
reg [7:0] zdo_r;

reg zmreq_r , ziorq_r , zwr_r , zrd_r , halt_r;

wire zcs = JWE|JOE;

always @(posedge CLK_n)
begin
  za_r    <= JA[15:0];

  if(zcs)
    zdo_r   <= JDO;

  if( (~zmreq_r || ~ziorq_r) && ~WAIT_n)
  begin
    // keep hold data
  end else begin
    zmreq_r <= ~(zcs & ~JA[16]); // MEM AREA 00000-0ffff
    ziorq_r <= ~(zcs &  JA[16]); // IO AREA  10000-1ffff
    zwr_r   <= ~JWE;
    zrd_r   <= ~JOE;
  end

  halt_r  <= JA[30];
  if(WAIT_n)
    zdi_r <= DI;
end

assign JDI = zdi_r;

assign M1_n   = 1'b0;
assign MREQ_n = zmreq_r;
assign IORQ_n = ziorq_r;
assign RD_n   = zrd_r;
assign WR_n   = zwr_r;
assign RFSH_n = 1'b1;
assign HALT_n = halt_r;
assign A      = za_r;

assign DO     = zdo_r;
assign DOE    = zwr_r;

endmodule
