/////////////////////////////////////////////////////////////////////////////
//
// FPGA Universal HID Bus Direct Switch I/F for TimePilot
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// Version : 1.00
//
// License:
//   This source is under the GPL license.
//
// Histry:
//   2004. 8.12 Added 2P START function
//   2004. 8.10 Bug Fix (K.Degawa)
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// pin assign
//
// HID[0] : Switch-Up
// HID[1] : Switch-Right
// HID[2] : Switch-Down
// HID[3] : Switch-Left
// HID[4] : Switch-Button1
// HID[5] : Switch-Button2
// HID[6] : LED1 , 3.3V open drain
// HID[7] : LED2 , 3.3V open drain
//
// Conbinational function
//
// +--------+---------------+---------------------+
// |Function|     Logic     |Function             |
// |        |UP |LT |DN |RT |                     |
// +--------+---+---+---+---+---------------------+
// |Reset   |ON |ON |ON |ON | allow all ON        |
// |Coin1   |ON |ON |OFF|ON | Left+Right + Up     |
// |Start1  |-- |ON |-- |OFF| Left                |
// +--------+---+---+---+---+---------------------+
//
/////////////////////////////////////////////////////////////////////////////

module hid_sw(
  CLK,
// Universal HID Bus
  HID_I,HID_O,HID_T,
// Internal Controll
  UP,DN,LT,RT,B1,B2,
  COIN,START,SERVICE,RESET_N,
// External Controll
  LED
);

input CLK;
input [7:0] HID_I;
output [7:0] HID_O;
output [7:0] HID_T;

output [1:0] UP,DN,LT,RT,B1,B2;
output [1:0] COIN,START;  //  2004.8.10 Bug Fix
output SERVICE,RESET_N;   //  2004.8.10 Bug Fix

input [1:0] LED;
/////////////////////////////////////////////////////////////////////////////
// Function
/////////////////////////////////////////////////////////////////////////////

// PL1
assign UP[0] = HID_I[0];
assign RT[0] = HID_I[1];
assign DN[0] = HID_I[2];
assign LT[0] = HID_I[3];
assign B1[0] = HID_I[4];
assign B2[0] = HID_I[5];
assign COIN[0]  = (LT[0]|RT[0]|UP[0]|~DN[0]);
assign START[0] = (LT[0]|~RT[0]);

// PL2
assign UP[1] = HID_I[0];
assign RT[1] = HID_I[1];
assign DN[1] = HID_I[2];
assign LT[1] = HID_I[3];
assign B1[1] = HID_I[4];
assign B2[1] = HID_I[5];
assign COIN[1]  = 1'b1;
assign START[1] = (~LT[0]|RT[0]);

wire SERVICE = 1'b1;
wire RESET_N = (HID_I[0]|HID_I[1]|HID_I[2]|HID_I[3]);

/////////////////////////////////////////////////////////////////////////////
// U HID Bus
/////////////////////////////////////////////////////////////////////////////
assign HID_T      = 8'b00111111;
assign HID_O[5:0] = 6'b000000;
assign HID_O[7:6] = LED[1:0];

endmodule
