/////////////////////////////////////////////////////////////////////////////
//
// FPGA Universal HID Bus PSX PAD I/F for TimePilot
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// Version : 1.00
//
// License:
//   This source is under the GPL license.
//
// Histry:
//   2004. 8.12 
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// HID pin assign
//
// HID[0] : psRXD(1) (pullup)
// HID[1] : psTXD(2)
// HID[2] : psSEL(6)
// HID[3] : psCLK(7)
// HID[4] : N.C.
// HID[5] : N.C.
// HID[6] : N.C.
// HID[7] : N.C.
//
// PAD assign
// +--------+------------------------------+
// |Function| Pad Assign                   |
// +--------+------------------------------+
// |1P allow| allow                        |
// |1P B1   | squeare                      |
// |1P B2   | x                            |
// |1P allow| allow                        |
// |1P B1   | squeare                      |
// |1P B2   | x                            |
// |1P ST   | start                        |
// |2P ST   | triangle                     |      
// |COIN1   | sel                          |
// |COIN2   | N.A.                         |
// |reset   | L1 + R1 + sel + start        |
// +--------+------------------------------+

/////////////////////////////////////////////////////////////////////////////

module hid_pspd(
  CLK,
// Universal HID Bus
  HID_I,HID_O,HID_T,
// Internal Controll
  UP,DN,LT,RT,B1,B2,
  COIN,START,SERVICE,RESET_N,
// External Controll
  LED
);

input CLK;
input [7:0] HID_I;
output [7:0] HID_O;
output [7:0] HID_T;

output [1:0] UP,DN,LT,RT,B1,B2;
output [1:0] COIN,START;  //  2004.8.10 Bug Fix
output SERVICE,RESET_N;   //  2004.8.10 Bug Fix

input [1:0] LED;
/////////////////////////////////////////////////////////////////////////////
// Function
/////////////////////////////////////////////////////////////////////////////

// 250KHz generator
reg [5:0] pris;
reg clk250k;

// 18.432MHz / 37 / 2 = 249KHz
always @(posedge CLK)
begin
  pris <= pris + 1;
  if(pris==36)
  begin
    clk250k <= ~clk250k;
    pris <= 0;
  end
end

wire psCLK,psTXD,psRXD,psSel;

wire [7:0] psd1;
 wire ps_lt = psd1[7];
 wire ps_dn = psd1[6];
 wire ps_rt = psd1[5];
 wire ps_up = psd1[4];
 wire ps_st = psd1[3];
 wire ps_rs = psd1[2];
 wire ps_ls = psd1[1];
 wire ps_se = psd1[0];
wire [7:0] psd2;
 wire ps_sq = psd2[7];
 wire ps_x  = psd2[6];
 wire ps_ci = psd2[5];
 wire ps_tr = psd2[4];
 wire ps_r1 = psd2[3];
 wire ps_l1 = psd2[2];
 wire ps_r2 = psd2[1];
 wire ps_l2 = psd2[0];

// PSPAD module
psPAD_top psPAD_top(
  .I_CLK250K(clk250k),
  .I_RSTn(1'b1),
  .O_psCLK(psCLK),.O_psSEL(psSEL),.O_psTXD(psTXD),.I_psRXD(psRXD),
  .O_RXD_1(psd1),
  .O_RXD_2(psd2),
  .O_RXD_3(), // Left-LR
  .O_RXD_4(), // Left-UD
  .O_RXD_5(), // Right-LR
  .O_RXD_6(), // Right-UD
  .I_CONF_SW(1'b0),
  .I_MODE_SW(1'b0),
  .I_MODE_EN(1'b0),
  .I_VIB_SW(2'b00),
  .I_VIB_DAT(8'h00)
);

// PL1
assign UP[0] = ps_up;
assign RT[0] = ps_rt;
assign DN[0] = ps_dn;
assign LT[0] = ps_lt;
assign B1[0] = ps_ci;
assign B2[0] = ps_x;
assign COIN[0]  = ps_se;
assign START[0] = ps_st;

// PL2
assign UP[1] = ps_up;
assign RT[1] = ps_rt;
assign DN[1] = ps_dn;
assign LT[1] = ps_lt;
assign B1[1] = ps_ci;
assign B2[1] = ps_x;
assign COIN[1]  = 1'b1;
assign START[1] = ps_sq;

wire SERVICE = 1'b1;
wire RESET_N = ps_l1 | ps_r1 | ps_se | ps_st;

/////////////////////////////////////////////////////////////////////////////
// U HID Bus
/////////////////////////////////////////////////////////////////////////////
assign HID_T      = 8'b11110001;
assign HID_O[7:4] = 4'b0000;

assign HID_O[3] = psCLK;
assign HID_O[2] = psSEL;
assign HID_O[1] = psTXD;
assign psRXD = HID_I[0];

endmodule
