/////////////////////////////////////////////////////////////////////////////
//
// "VGA 15BPP" video encoder
//
// Version : 1.10
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// License:
//   This source is under the GPL license.
//
// Histry:
//   2004. 7.21 1st Release
//   2004. 8.12 Change Primitive Xilinx BSRAM to generic dpram
//
/////////////////////////////////////////////////////////////////////////////

module venc_vga(
// Timming Signal
  pclk,dclk,vclk,hs_in,vs_in,
  pal_ai,pal_eni,
  pal_ao,pal_eno,
  r_in , g_in , b_in,
  r_out,r_tri,g_out,g_tri,b_out,b_tri,         
  hs_out,hs_tri,vs_out,vs_tri
);

// Timming Signal
input pclk;             // 6.144MHz  input pixel clock
input dclk;             // 12.288Mhz output pixel clock
input vclk;		// VIDEO pixel clock (not used)
input hs_in;            // HSYNC input (16KHz)
input vs_in;            // VSYNC input (60Hz)

// PrePalette Conversion
input [4:0] pal_ai;
input pal_eni;
output [4:0] pal_ao;
output pal_eno;

// Post Palette Encode
input [4:0] r_in , g_in , b_in;
output [4:0] r_out,g_out,b_out; // R,G,B output level
output [4:0] r_tri,g_tri,b_tri; // R,G,B tristate control
output hs_out;          // HSYNC output
output hs_tri;          // HSYNC tristate control
output vs_out;          // VSYNC output
output vs_tri;          // VSYNC tristate control

//---------------------------------------------------------------------------
// setup parameter
//---------------------------------------------------------------------------

parameter H_COUNT = 384; // number of pixels in H-SCAN
parameter HS_POS  = 16;  // HSYNC position 
parameter HS_COUNT= 8;   // HSYNC width / pixel
parameter VS_COUNT= 8;   // VSYNC width / HSYNC_OUT

//---------------------------------------------------------------------------
// dualport line buffer
//---------------------------------------------------------------------------
reg [5:0] line_buf[511:0];
reg [5:0] pal_out6;

//---------------------------------------------------------------------------
// input timming
//---------------------------------------------------------------------------
reg [8:0] hpos_i;    // input capture postion
reg ihs;
wire ic_raise = ~ihs&hs_in; // raise hsync
always @(posedge pclk)
begin
	hpos_i <= ic_raise ? 0 : hpos_i + 1;
	ihs =  hs_in;

        // store line buf
	line_buf[hpos_i] <= {pal_eni,pal_ai};
end

//---------------------------------------------------------------------------
//output timming
//---------------------------------------------------------------------------
reg [8:0] hpos_o;
reg ohs , hsync_o;
wire oh_raise = ~ohs&hs_in; // raise hsync 
wire oh_return = oh_raise | (hpos_o == H_COUNT-1);
always @(posedge dclk)
begin
	hpos_o <= oh_return ? 0 : hpos_o + 1;
	ohs <= hs_in;

	// remake HSYNC
	if(hpos_o == HS_POS)
	begin
		hsync_o <= 1'b1;
	end
	if(hpos_o == (HS_POS+HS_COUNT) )
		hsync_o <= 1'b0;

	// get from line buffer
	pal_out6 <= line_buf[hpos_o];
end

//---------------------------------------------------------------------------
// vsync remake
//
// 1 HSYNC_IN delay & HSYNC pulse width = 4xHSYNC(in)
//---------------------------------------------------------------------------

reg [2:0] vs_cnt;
reg vsync_o;

always @(posedge hsync_o)
begin
	if(~vs_in)
	begin
		vs_cnt <= VS_COUNT-1;
		vsync_o <= 1;
	end else begin
		if(vs_cnt==0)
			vsync_o <= 0;
 	   	vs_cnt <= vs_cnt - 1;
	end
end

//---------------------------------------------------------------------------
// output
//---------------------------------------------------------------------------

assign r_tri = { 5'b00000 };
assign r_out = { r_in };
assign g_tri = { 5'b00000 };
assign g_out = { g_in };
assign b_tri = { 5'b00000 };
assign b_out = { b_in };

// converted SYNC signal
assign vs_tri = 1'b0;
assign vs_out = ~vsync_o;
assign hs_tri = 1'b0;
assign hs_out = ~hsync_o;

assign pal_ao  = pal_out6[4:0];
assign pal_eno = pal_out6[5];

endmodule
