/////////////////////////////////////////////////////////////////////////////
//
// FPGA TimePilot TopDesign
//
// Version : 0.90
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// License:
//   This source is under the GPL license.
//
// Histry:
//   2004. 8.14 moved Xilinx primitive into compile switch (by K.Dwgawa)
//   2004. 8.10 Bug Fix on Quartus2 v4.0sp1 (by K.Dwgawa)
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

// ----- option switch -----
`include "src/switch.vh"

// ---- default set -----

//`define SPARTAN2_IO // Xilinx SpartanII , SpartanIIE
//`define CYCLONE_IO // Altera Cyclone

`ifndef ROM_MSB
  `define ROM_MSB  18 // 4MBit
`endif

`ifndef ROM_BANK
  `define ROM_BANK 3'b000 // 00000-0ffff
`endif

`ifndef VENC_MODULE
  `define VENC_MODULE venc_rgb  // 15bpp 16KHz RGB
`endif

`ifndef SENC_MODULE
  `define SENC_MODULE senc_sd   // SigmaDelta Analog
`endif

`ifndef HID_MODULE
  `define HID_MODULE hid_sw    // direct switch
`endif

`ifndef SUB_MODULE
  `define SUB_MODULE tp_sub    // sub board
`endif

`ifndef IMEM_MODULE
  `define IMEM_MODULE imem_gen // Generic RAM
`endif

//
`ifndef DSW1_SET
  `define DSW1_SET 8'hff
`endif

`ifndef DSW2_SET
  `define DSW2_SET 8'hff
`endif

module fpga_tpx(
// 18.432MHz generic CLOCK input
	CLK18M432,
// Universal Video I/F
	VCLK,
	VGA_R,VGA_G,VGA_B,
	n_HSYNC,n_VSYNC,
//ROM interface
	RA,RD,RCS,ROE,RWE,
//Sound
	DAC_L,DAC_R,
//Universal HID Bus
	HID
);

/////////////////////////////////////////////////////////////////////////////
// module I/O assign
/////////////////////////////////////////////////////////////////////////////

// 18.432MHz generic CLOCK input
input CLK18M432; // 18.432MHz

// Universal Video I/F
input  VCLK;
output [4:0] VGA_R , VGA_G , VGA_B;
output n_HSYNC,n_VSYNC;

// Universal HID Bus
inout [7:0] HID;

// External PROM interface
output [`ROM_MSB:0] RA;
inout [7:0] RD;
output RCS,ROE,RWE;

//Sound Output
output DAC_L,DAC_R;

/////////////////////////////////////////////////////////////////////////////
// TimePilot Main+I/O Circuit
/////////////////////////////////////////////////////////////////////////////

wire pclk,pclk2x;
wire [4:0] red,green,blue;
wire hsync;
wire vsync;

// Controll
wire [1:0] UP,LT,RT,DN,B1,CO,ST;
wire SV;
wire usr_rst_n;

// Sound I/F
wire [7:0] snd_do;
wire scmd_w , sirq_w;

// Timming
wire [2:0] phase_3p;
wire [8:0] hcnt;

// Main CPU ROM
wire [14:0] m_a;
wire [7:0] mrom_do;
wire mrom_as;
wire [7:0] m_wd; // CPU write data

// Tilemap Pattern ROM
wire [12:0] tpat_a;  // tile pixel address
wire [7:0] tpat_d;

// Object Pattern ROM
wire [13:0] opat_a; // obj pixel address
wire [7:0] opat_d;

// obj ram
wire [7:0] oram_a;
wire [7:0] oram0_di , oram1_di;
wire oram0_we , oram1_we;

// viode & work RAM I/F
wire [11:0] tram_a;
wire [7:0] tram_dr;
wire tile_we;
wire tile_cs;
// line ram
wire [7:0] lram0_a,lram1_a;
wire [3:0] lram0_do,lram1_do;
wire [3:0] lram0_di,lram1_di;
wire lram0_we,lram1_we;

//wire [7:0] pram_di,pram_do;
//wire pram_we;
wire [7:0] tclut_a;
wire [3:0] tclut_d;

// object clut ROM
wire [7:0] oclut_a;
wire [3:0] oclut_d;

//palette ROM
wire [4:0] pal_a;
wire pal_en;
wire [14:0] pal_d;

// video encoded palette rom
wire [4:0] pal_a_e;
wire pal_en_e;

tp_main TimePilotMain(
// Clock
  .RST_N(usr_rst_n),
  .CLK18M432(CLK18M432),
// Video
  .pclk(pclk),
  .dclk(pclk2x),
  .R(red),.G(green),.B(blue),
  .hsync(hsync),.vsync(vsync),
// Controller
  .UP(UP),.DN(DN),.LT(LT),.RT(RT),.B1(B1),.B2(2'b11),.B3(2'b11),
  .CO(CO),.ST(ST),.SV(SV),
// DIP switch
  .DSW1(`DSW1_SET),.DSW2(`DSW2_SET),
// Sound I/F
  .M_DO(snd_do),.SCMD_W(scmd_w),.SIRQ_W(sirq_w),
// State
  .PHASE3x(phase_3p),.HCNT(hcnt),

// Main CPU ROM
  .M_A(m_a),.MROM_DI(mrom_do),.MROM_OE(mrom_as),//Bug Fix 2004.8.10
// Tilemap Pattern ROM
  .TROM_A(tpat_a),.TROM_DI(tpat_d),
// Object Pattern ROM
  .OROM_A(opat_a),.OROM_DI(opat_d),

// TileMap RAM & OBJect RAM Write data
  .M_WD(m_wd),
// TileMap RAM
  .TRAM_A(tram_a),.TRAM_DI(tram_dr),
  .TRAM_CS(tile_cs),.TRAM_WE(tile_we),
// TileMap ColorLookUpTable ROM
  .TCLUT_A(tclut_a),.TCLUT_DI(tclut_d),
// Object RAM
  .OBJ_A(oram_a),
  .OBJ0_DI(oram0_di),.OBJ1_DI(oram1_di),
  .OBJ0_WE(oram0_we),.OBJ1_WE(oram1_we),
//Object ColorLookUpTable ROM
  .OCLUT_A(oclut_a),.OCLUT_DI(oclut_d),
// Object Line RAM
  .L0A(lram0_a),.L0DI(lram0_di),.L0DO(lram0_do),.L0WE(lram0_we),
  .L1A(lram1_a),.L1DI(lram1_di),.L1DO(lram1_do),.L1WE(lram1_we),
// Palette ROM
  .PAL_A(pal_a),.PAL_EN(pal_en),.PAL_DI(pal_d)
);

/////////////////////////////////////////////////////////////////////////////
// Sub CPU Clock Generator
/////////////////////////////////////////////////////////////////////////////
wire subclk , clk17m8;
sub_clk sub_clk(.CLK18M432(CLK18M432),.CLK1M78975(subclk));

`ifdef SPARTAN2_IO
  // Xilinx SpartanII
  BUFG SUB_CLOCK(.I(subclk),.O(clk17m8));
`elsif CYCLONE_IO
  // altera cyclone
  assign clk17m8 = subclk;
`else
  // standard syntax
  assign clk17m8 = subclk;
`endif


/////////////////////////////////////////////////////////////////////////////
// TimePilot Sub (sound) Circuit
/////////////////////////////////////////////////////////////////////////////

// Sub CPU ROM / RAM
wire [12:0] smem_a;
wire [7:0] srom_d , sram_dr ,sram_dw;
wire srom_oe , sram_cs , sram_we;
// Sound
wire [9:0] pcm10;


`SUB_MODULE TimePilotSub(
// System
  .rst_n(usr_rst_n),
  .clk17m8(clk17m8),
// Main I/F
  .snd_di(snd_do), .snd_cmd_w(scmd_w), .snd_irq_w(sirq_w),
// ROM/RAM
  .mem_a(smem_a),
  .rom_oe(srom_oe), .rom_di(srom_d),
  .ram_cs(sram_cs), .ram_we(sram_we), .ram_di(sram_dr), .ram_do(sram_dw),
// sound I/F
  .snd_out(pcm10)
);

/////////////////////////////////////////////////////////////////////////////
// internal memory 
/////////////////////////////////////////////////////////////////////////////

// SubCPU Work RAM
// TileMap RAM & OBJect RAM Write data
// TileMap RAM
// TileMap ColorLookUpTable ROM
// Object RAM
// Object ColorLookUpTable ROM
// Object Line RAM
// Palette ROM
//
// configration internal ROM from external PROM

// config Access
wire cfg_en;
wire [9:0] cfg_a;
wire [7:0] cfg_d;

// configration ROM address
// 6000-601f palette H
// 6020-603f palette L
// 6100-61ff tilemap clut
// 6200-63ff object clut

`IMEM_MODULE InternalMemory(
  .rst_n(usr_rst_n),
  .CLK_3x(CLK18M432), .PHASE_3x(phase_3p),
// SubCPU Work RAM
  .SA(smem_a[9:0]),.SDO(sram_dr),.SDI(sram_dw),.SCS(sram_cs),.SWE(sram_we),
// TileMap RAM & OBJect RAM Write data
  .V_DI(m_wd),
// TileMap RAM
  .TRAM_A(tram_a),.TRAM_DO(tram_dr),
  .TRAM_CS(tile_cs),.TRAM_WE(tile_we),
// TileMap ColorLookUpTable ROM
  .TCLUT_A(tclut_a),.TCLUT_DO(tclut_d),
// Object RAM
  .OBJ_A(oram_a),
  .OBJ0_DO(oram0_di),.OBJ1_DO(oram1_di),
  .OBJ0_WE(oram0_we),.OBJ1_WE(oram1_we),
//Object ColorLookUpTable ROM
  .OCLUT_A(oclut_a),.OCLUT_DO(oclut_d),
// Object Line RAM
  .L0A(lram0_a),.L0DO(lram0_di),.L0DI(lram0_do),.L0WE(lram0_we),
  .L1A(lram1_a),.L1DO(lram1_di),.L1DI(lram1_do),.L1WE(lram1_we),
// Palette ROM
//  .PAL_A(pal_a),.PAL_EN(pal_en),.PAL_DO(pal_d),
  .PAL_A(pal_a_e),.PAL_EN(pal_en_e),.PAL_DO(pal_d),
// config intROM from extPROM
  .CFG_EN(cfg_en),.CFG_A(cfg_a),.CFG_DI(cfg_d)
);

/////////////////////////////////////////////////////////////////////////////
// External Memory
// interleave CPU address & video pattern address
/////////////////////////////////////////////////////////////////////////////

// Main CPU ROM
// Sub CPU ROM
// Tilemap Pattern ROM
// Object Pattern ROM / Config Internal ROM

// ROM
wire [15:0] mem_a;
wire [7:0]  mem_di;
wire [7:0]  mem_do;
wire mem_we;
wire mem_wo;
wire mem_oe;

extmem ExternalMemory(
// Clock
  .CLK3x(CLK18M432), .PHASE3x(phase_3p),
  .HCNT(hcnt[1:0]),
// Static Memory I/F
  .MEM_A(mem_a),.MEM_DI(mem_di),.MEM_DO(mem_do),
  .MEM_WE(mem_we),.MEM_WO(mem_wo),.MEM_OE(mem_oe),
// Main CPU ROM
  .MROM_A(m_a),.MROM_DO(mrom_do),.MROM_OE(mrom_as),// Bug Fix 2004.8.10
// Sub CPU ROM
  .SROM_A(smem_a),.SROM_DO(srom_d),.SROM_OE(srom_oe),
// Tilemap Pattern ROM
  .TROM_A(tpat_a),.TROM_DO(tpat_d),
// Object Pattern ROM
  .OROM_A(opat_a),.OROM_DO(opat_d),
// Config IntROM
  .CFG_EN(cfg_en),.CFG_A(cfg_a),.CFG_DO(cfg_d)
);

/////////////////////////////////////
// Pin Assign
/////////////////////////////////////
`ifdef SPARTAN2_IO
// Xilinx SpartanII
assign RA  = {`ROM_BANK,mem_a};
IOBUF_S_2 PRD0(.IO(RD[0]),.I(mem_do[0]),.O(mem_di[0]),.T(~mem_wo));
IOBUF_S_2 PRD1(.IO(RD[1]),.I(mem_do[1]),.O(mem_di[1]),.T(~mem_wo));
IOBUF_S_2 PRD2(.IO(RD[2]),.I(mem_do[2]),.O(mem_di[2]),.T(~mem_wo));
IOBUF_S_2 PRD3(.IO(RD[3]),.I(mem_do[3]),.O(mem_di[3]),.T(~mem_wo));
IOBUF_S_2 PRD4(.IO(RD[4]),.I(mem_do[4]),.O(mem_di[4]),.T(~mem_wo));
IOBUF_S_2 PRD5(.IO(RD[5]),.I(mem_do[5]),.O(mem_di[5]),.T(~mem_wo));
IOBUF_S_2 PRD6(.IO(RD[6]),.I(mem_do[6]),.O(mem_di[6]),.T(~mem_wo));
IOBUF_S_2 PRD7(.IO(RD[7]),.I(mem_do[7]),.O(mem_di[7]),.T(~mem_wo));
OBUF_S_2 PRCS(.O(RCS),.I(1'b0));
OBUF_S_2 PRWE(.O(RWE),.I(1'b1));
OBUF_S_2 PROE(.O(ROE),.I(1'b0));
`elsif CYCLONE_IO
// altera cyclone
assign RA  = {`ROM_BANK,mem_a}; // 64K depth
assign RD  = mem_wo ? mem_do : 8'bzzzzzzzz;
assign mem_di = RD;
assign RCS = 1'b0;
assign RWE = 1'b1;
assign ROE = 1'b0;
`else
// standard syntax
assign RA  = {`ROM_BANK,mem_a}; // 64K depth
assign RD  = mem_wo ? mem_do : 8'bzzzzzzzz;
assign mem_di = RD;
assign RCS = 1'b0;
assign RWE = 1'b1;
assign ROE = 1'b0;
`endif

/////////////////////////////////////////////////////////////////////////////
// HumanInterface
/////////////////////////////////////////////////////////////////////////////
wire [7:0] hid_i , hid_o , hid_t;

`HID_MODULE HumanInterfaceDevice(
  .CLK(CLK18M432),
// Universal HID Bus
  .HID_I(hid_i),.HID_O(hid_o),.HID_T(hid_t),
// Internal Controll
  .UP(UP),.DN(DN),.LT(LT),.RT(RT),.B1(B1),.B2(),
  .COIN(CO),.START(ST),.SERVICE(SV),.RESET_N(usr_rst_n),
// External Controll
  .LED(2'b11)
);

/////////////////////////////////////
// Pin Assign
/////////////////////////////////////
`ifdef SPARTAN2_IO
// Xilinx SpartanII
IOBUF_S_2 PHID0(.IO(HID[0]),.I(hid_o[0]),.O(hid_i[0]),.T(hid_t[0]));
IOBUF_S_2 PHID1(.IO(HID[1]),.I(hid_o[1]),.O(hid_i[1]),.T(hid_t[1]));
IOBUF_S_2 PHID2(.IO(HID[2]),.I(hid_o[2]),.O(hid_i[2]),.T(hid_t[2]));
IOBUF_S_2 PHID3(.IO(HID[3]),.I(hid_o[3]),.O(hid_i[3]),.T(hid_t[3]));
IOBUF_S_2 PHID4(.IO(HID[4]),.I(hid_o[4]),.O(hid_i[4]),.T(hid_t[4]));
IOBUF_S_2 PHID5(.IO(HID[5]),.I(hid_o[5]),.O(hid_i[5]),.T(hid_t[5]));
IOBUF_S_2 PHID6(.IO(HID[6]),.I(hid_o[6]),.O(hid_i[6]),.T(hid_t[6]));
IOBUF_S_2 PHID7(.IO(HID[7]),.I(hid_o[7]),.O(hid_i[7]),.T(hid_t[7]));
`elsif CYCLONE_IO
// altera cyclone
assign HID[0] = hid_t[0] ? 1'bz : hid_o[0];
assign HID[1] = hid_t[1] ? 1'bz : hid_o[1];
assign HID[2] = hid_t[2] ? 1'bz : hid_o[2];
assign HID[3] = hid_t[3] ? 1'bz : hid_o[3];
assign HID[4] = hid_t[4] ? 1'bz : hid_o[4];
assign HID[5] = hid_t[5] ? 1'bz : hid_o[5];
assign HID[6] = hid_t[6] ? 1'bz : hid_o[6];
assign HID[7] = hid_t[7] ? 1'bz : hid_o[7];
assign hid_i = HID;
`else
// standard syntax
assign HID[0] = hid_t[0] ? 1'bz : hid_o[0];
assign HID[1] = hid_t[1] ? 1'bz : hid_o[1];
assign HID[2] = hid_t[2] ? 1'bz : hid_o[2];
assign HID[3] = hid_t[3] ? 1'bz : hid_o[3];
assign HID[4] = hid_t[4] ? 1'bz : hid_o[4];
assign HID[5] = hid_t[5] ? 1'bz : hid_o[5];
assign HID[6] = hid_t[6] ? 1'bz : hid_o[6];
assign HID[7] = hid_t[7] ? 1'bz : hid_o[7];
assign hid_i = HID;
`endif



/////////////////////////////////////////////////////////////////////////////
// Sound encoder
/////////////////////////////////////////////////////////////////////////////
wire dac_out;

`SENC_MODULE sound_encoder(
  .CLK(CLK18M432),
  .IN(pcm10),
  .OUT(dac_out)
);

/////////////////////////////////////
// Pin Assign
/////////////////////////////////////
`ifdef SPARTAN2_IO
// Xilinx SpartanII
OBUF_F_24 DACL (.I(dac_out), .O(DAC_L));
OBUF_F_24 DACR (.I(dac_out), .O(DAC_R));
`elsif CYCLONE_IO
// altera cyclone
assign DAC_L = dac_out;
assign DAC_R = dac_out;
`else
// standard syntax
assign DAC_L = dac_out;
assign DAC_R = dac_out;
`endif

/////////////////////////////////////////////////////////////////////////////
// Video encoder & output
/////////////////////////////////////////////////////////////////////////////

// external video clock
wire video_clock;
`ifdef SPARTAN2_IO   //Bug Fix 2004.8.14
IBUFG video_clock_i(.I(VCLK),.O(video_clock));
`else
assign video_clock = VCLK;
`endif

wire hs_out , vs_out;
wire hs_tri , vs_tri;
wire [4:0] r_out,g_out,b_out;
wire [4:0] r_tri,g_tri,b_tri;

`VENC_MODULE video_encoder(
  .pclk(pclk),
  .dclk(pclk2x),
  .vclk(video_clock),
  .hs_in(hsync),
  .vs_in(vsync),
//
  .pal_ai(pal_a),.pal_eni(pal_en),
  .pal_ao(pal_a_e),.pal_eno(pal_en_e),
//
  .r_in(red),.g_in(green),.b_in(blue),
  .r_out(r_out), .r_tri(r_tri),
  .g_out(g_out), .g_tri(g_tri),
  .b_out(b_out), .b_tri(b_tri),
  .hs_tri(hs_tri),.hs_out(hs_out),
  .vs_tri(vs_tri),.vs_out(vs_out)
);

/////////////////////////////////////
// Pin Assign
/////////////////////////////////////
`ifdef SPARTAN2_IO
// Xilinx SpartanII
OBUFT_S_24 VR4(.O(VGA_R[0]),.I(r_out[4]),.T(r_tri[4]));
OBUFT_S_24 VR3(.O(VGA_R[1]),.I(r_out[3]),.T(r_tri[3]));
OBUFT_S_24 VR2(.O(VGA_R[2]),.I(r_out[2]),.T(r_tri[2]));
OBUFT_S_24 VR1(.O(VGA_R[3]),.I(r_out[1]),.T(r_tri[1]));
OBUFT_S_24 VR0(.O(VGA_R[4]),.I(r_out[0]),.T(r_tri[0]));

OBUFT_S_24 VG4(.O(VGA_G[0]),.I(g_out[4]),.T(g_tri[4]));
OBUFT_S_24 VG3(.O(VGA_G[1]),.I(g_out[3]),.T(g_tri[3]));
OBUFT_S_24 VG2(.O(VGA_G[2]),.I(g_out[2]),.T(g_tri[2]));
OBUFT_S_24 VG1(.O(VGA_G[3]),.I(g_out[1]),.T(g_tri[1]));
OBUFT_S_24 VG0(.O(VGA_G[4]),.I(g_out[0]),.T(g_tri[0]));

OBUFT_S_24 VB4(.O(VGA_B[0]),.I(b_out[4]),.T(b_tri[4]));
OBUFT_S_24 VB3(.O(VGA_B[1]),.I(b_out[3]),.T(b_tri[3]));
OBUFT_S_24 VB2(.O(VGA_B[2]),.I(b_out[2]),.T(b_tri[2]));
OBUFT_S_24 VB1(.O(VGA_B[3]),.I(b_out[1]),.T(b_tri[1]));
OBUFT_S_24 VB0(.O(VGA_B[4]),.I(b_out[0]),.T(b_tri[0]));

OBUFT_S_2 HS_DRIVER(.O(n_HSYNC),.I(hs_out),.T(hs_tri));
OBUFT_S_2 VS_DRIVER(.O(n_VSYNC),.I(vs_out),.T(vs_tri));
`elsif CYCLONE_IO
// altera cyclone
assign VGA_R[0] = r_tri[4] ? 1'bz : r_out[4];
assign VGA_R[1] = r_tri[3] ? 1'bz : r_out[3];
assign VGA_R[2] = r_tri[2] ? 1'bz : r_out[2];
assign VGA_R[3] = r_tri[1] ? 1'bz : r_out[1];
assign VGA_R[4] = r_tri[0] ? 1'bz : r_out[0];

assign VGA_G[0] = g_tri[4] ? 1'bz : g_out[4];
assign VGA_G[1] = g_tri[3] ? 1'bz : g_out[3];
assign VGA_G[2] = g_tri[2] ? 1'bz : g_out[2];
assign VGA_G[3] = g_tri[1] ? 1'bz : g_out[1];
assign VGA_G[4] = g_tri[0] ? 1'bz : g_out[0];

assign VGA_B[0] = b_tri[4] ? 1'bz : b_out[4];
assign VGA_B[1] = b_tri[3] ? 1'bz : b_out[3];
assign VGA_B[2] = b_tri[2] ? 1'bz : b_out[2];
assign VGA_B[3] = b_tri[1] ? 1'bz : b_out[1];
assign VGA_B[4] = b_tri[0] ? 1'bz : b_out[0];

assign n_HSYNC  = hs_tri ? 1'bz : hs_out;
assign n_VSYNC  = vs_tri ? 1'bz : vs_out;
`else
// standard syntax
assign VGA_R[0] = r_tri[4] ? 1'bz : r_out[4];
assign VGA_R[1] = r_tri[3] ? 1'bz : r_out[3];
assign VGA_R[2] = r_tri[2] ? 1'bz : r_out[2];
assign VGA_R[3] = r_tri[1] ? 1'bz : r_out[1];
assign VGA_R[4] = r_tri[0] ? 1'bz : r_out[0];

assign VGA_G[0] = g_tri[4] ? 1'bz : g_out[4];
assign VGA_G[1] = g_tri[3] ? 1'bz : g_out[3];
assign VGA_G[2] = g_tri[2] ? 1'bz : g_out[2];
assign VGA_G[3] = g_tri[1] ? 1'bz : g_out[1];
assign VGA_G[4] = g_tri[0] ? 1'bz : g_out[0];

assign VGA_B[0] = b_tri[4] ? 1'bz : b_out[4];
assign VGA_B[1] = b_tri[3] ? 1'bz : b_out[3];
assign VGA_B[2] = b_tri[2] ? 1'bz : b_out[2];
assign VGA_B[3] = b_tri[1] ? 1'bz : b_out[1];
assign VGA_B[4] = b_tri[0] ? 1'bz : b_out[0];

assign n_HSYNC  = hs_tri ? 1'bz : hs_out;
assign n_VSYNC  = vs_tri ? 1'bz : vs_out;
`endif

endmodule
