/////////////////////////////////////////////////////////////////////////////
//
// Xilinx SpartanII/E Internal Memory for TimePilot
//
// Version : 1.00
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// License:
//   This source is under the GPL license.
//
// Histry:
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// internal memory
/////////////////////////////////////////////////////////////////////////////

module imem_xc2(
  rst_n,
//
  CLK_3x, PHASE_3x,
// SubCPU Memory
  SA,SDO,SDI,SCS,SWE,
// Tilemap RAM & OBJect RAM Write data
  V_DI,
// TileMap RAM
  TRAM_A,TRAM_DO,TRAM_CS,TRAM_WE,
// TileMap ColorLookUpTable ROM
  TCLUT_A,TCLUT_DO,

// Object RAM
  OBJ_A,OBJ0_DO,OBJ0_WE,OBJ1_DO,OBJ1_WE,
// Object Line RAM
  L0A,L0DO,L0DI,L0WE,
  L1A,L1DO,L1DI,L1WE,
// OBJ ColorLookUpTable ROM
  OCLUT_A,OCLUT_DO,

// Palette ROM
  PAL_A,PAL_EN,PAL_DO,

// Configration ROM write
  CFG_EN,CFG_A, CFG_DI
);

/////////////////////////////////////////////////////////////////////////////
// module I/O assign
/////////////////////////////////////////////////////////////////////////////
input rst_n;
input CLK_3x;       // x3 pixel clock
input [2:0] PHASE_3x; // x3 clock phase signal

// SubCPU Work RAM
input [9:0] SA;
output [7:0] SDO;
input [7:0] SDI;
input SCS;
input SWE;

// TileMap RAM & OBJect RAM Write data
input [7:0] V_DI;

// TileMap RAM
input [11:0] TRAM_A;
output [7:0] TRAM_DO;
input TRAM_CS,TRAM_WE;

// TileMap ColorLookUpTable ROM
input [7:0] TCLUT_A;
output [3:0] TCLUT_DO;

// Object RAM
input [7:0] OBJ_A;
output [7:0] OBJ0_DO,OBJ1_DO;
input OBJ0_WE,OBJ1_WE;

//Object ColorLookUpTable ROM
input [7:0] OCLUT_A;
output [3:0] OCLUT_DO;

// Object Line RAM
input [7:0] L0A,L1A;
input [3:0] L0DI,L1DI;
output [3:0] L0DO,L1DO;
input L0WE,L1WE;

// Palette ROM
input [4:0] PAL_A;
input PAL_EN;
output [14:0] PAL_DO;

// Configration Bus
output CFG_EN;
output [9:0] CFG_A;
input [7:0] CFG_DI;

/////////////////////////////////////////////////////////////////////////////
// Internal TCLUT & OCLUT PAL1 & PLA2 pre-load circuit
/////////////////////////////////////////////////////////////////////////////
reg [9:0] cfg_a_r; // 000-23f
reg [1:0] wcnt;    // 1.5MHz
wire cfg_busy = (cfg_a_r!=10'h240);

always @(posedge CLK_3x or negedge rst_n)
begin
  if(~rst_n)
  begin
    wcnt  <= 0;
    cfg_a_r <= 0;
  end else begin
    if(cfg_busy & PHASE_3x[2])
      {cfg_a_r,wcnt} <= {cfg_a_r,wcnt} + 1;
  end
end
assign CFG_EN = cfg_busy;
assign CFG_A  = cfg_a_r;

// address decode
wire tclut_we = cfg_busy & ~cfg_a_r[9];             // 000-0ff TCLUT
                                                    // 100-1ff OCLUT
wire pal_we1 = cfg_busy & cfg_a_r[9] & ~cfg_a_r[5]; // 200-21f PAL H
wire pal_we2 = cfg_busy & cfg_a_r[9] &  cfg_a_r[5]; // 220-23f PLA L

/////////////////////////////////////////////////////////////////////////////
// Sub CPU SRAM
/////////////////////////////////////////////////////////////////////////////
RAMB4_S4 SUB_RAM0(.CLK(CLK_3x),.ADDR(SA),.DO(SDO[7:4]),.DI(SDI[7:4]),.EN(1'b1),.WE(SWE),.RST(~SCS));
RAMB4_S4 SUB_RAM1(.CLK(CLK_3x),.ADDR(SA),.DO(SDO[3:0]),.DI(SDI[3:0]),.EN(1'b1),.WE(SWE),.RST(~SCS));

/////////////////////////////////////////////////////////////////////////////
// Video RAM
/////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////
// 1.Tilemap RAM
///////////////////////////////////////
RAMB4_S1 TRAM0 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[0]),.DI(V_DI[0]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));
RAMB4_S1 TRAM1 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[1]),.DI(V_DI[1]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));
RAMB4_S1 TRAM2 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[2]),.DI(V_DI[2]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));
RAMB4_S1 TRAM3 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[3]),.DI(V_DI[3]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));
RAMB4_S1 TRAM4 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[4]),.DI(V_DI[4]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));
RAMB4_S1 TRAM5 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[5]),.DI(V_DI[5]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));
RAMB4_S1 TRAM6 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[6]),.DI(V_DI[6]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));
RAMB4_S1 TRAM7 (.CLK(CLK_3x),.ADDR(TRAM_A),.DO(TRAM_DO[7]),.DI(V_DI[7]),.EN(1'b1),.WE(TRAM_WE),.RST(~TRAM_CS));

///////////////////////////////////////
// 4.Object RAM x2
///////////////////////////////////////
wire [8:0] oram_a9  =  { 1'b0,OBJ_A };
wire [8:0] oram_b9  =  { 1'b1,OBJ_A };

RAMB4_S8_S8 oram (
  .CLKA(CLK_3x),.ADDRA(oram_a9),.DOA(OBJ0_DO),.DIA(V_DI),.WEA(OBJ0_WE),.ENA(1'b1),.RSTA(1'b0),
  .CLKB(CLK_3x),.ADDRB(oram_b9),.DOB(OBJ1_DO),.DIB(V_DI),.WEB(OBJ1_WE),.ENB(1'b1),.RSTB(1'b0)  // PRAM
);

///////////////////////////////////////
// Object Line RAM x2
///////////////////////////////////////
wire [9:0] la0_10 = { 2'b00, L0A };
wire [9:0] la1_10 = { 2'b01, L1A };

RAMB4_S4_S4 lram2 (
  .CLKA(CLK_3x),.ADDRA(la0_10),.DOA(L0DO),.DIA(L0DI),.WEA(L0WE),.ENA(1'b1),.RSTA(1'b0),
  .CLKB(CLK_3x),.ADDRB(la1_10),.DOB(L1DO),.DIB(L1DI),.WEB(L1WE),.ENB(1'b1),.RSTB(1'b0)
);

///////////////////////////////////////
// Tilemap ColorLookUpTable ROM(RAM)
// Object ColorLookUpTable ROM(RAM)
///////////////////////////////////////
wire [9:0] tc_a10 = cfg_busy ? {1'b0,cfg_a_r[8:0]} : { 2'b00,TCLUT_A };
wire [9:0] oc_a10 = { 2'b01,OCLUT_A };

RAMB4_S4_S4 clut_rom00 (
	.CLKA(CLK_3x),.ADDRA(tc_a10),.DOA(TCLUT_DO),.DIA(CFG_DI[3:0]),.WEA(tclut_we),.ENA(1'b1),.RSTA(1'b0),
	.CLKB(CLK_3x),.ADDRB(oc_a10),.DOB(OCLUT_DO),.DIB(4'h0),.WEB(1'b0),.ENB(1'b1),.RSTB(1'b0)
);


///////////////////////////////////////
// Palette ROM(RAM)
///////////////////////////////////////
reg [7:0] pal_ram0[31:0];
reg [6:0] pal_ram1[31:0];

wire [4:0] pal_ma = cfg_busy ? cfg_a_r[4:0] : PAL_A;

always @(posedge CLK_3x)
begin
  if(pal_we1) pal_ram0[pal_ma] <= CFG_DI;
  if(pal_we2) pal_ram1[pal_ma] <= CFG_DI[7:1];
end

// b = [14:10] , g = [ 9: 5] , r = [ 4: 0]
wire [14:0] pal_rom = {pal_ram0[pal_ma],pal_ram1[pal_ma]};
assign PAL_DO = pal_rom & {15{PAL_EN}};

endmodule
