/////////////////////////////////////////////////////////////////////////////
//
// TilePilot MAIN addredd decoder
//
// Version : 1.00
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// It isn't suitable to put this source on GPL. 
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source include this document.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

module tp_adec(
// CPU
  A, D0I, n_MREQ, n_RD, n_RFSH,
// Memory Area
  ROM, TRAM, OBJ0, OBJ1,
// Output Port
  SOUND, WDT, IRQEN, FLIP, IRQTRG, COIN1, COIN2,
// Input Port
  SCAN, DSW2, IN0, IN1, IN2, DSW1
);

input [15:0] A;
input D0I;
input n_MREQ;
input n_RD;
input n_RFSH;

// Memory Area
output ROM;  // 0000-7fff
output TRAM; // a000-afff
output OBJ0; // b000-b3ff
output OBJ1; // b400-b7ff

// Output Port
output SOUND;  // c000.W
output WDT;    // c200.W
output IRQEN;  // c300.W
output FLIP;   // c302.W
output IRQTRG; // c304.W
output COIN1;  // c30a.W
output COIN2;  // c30c.W

output SCAN;  // c000.R
output IN0;   // c200.R
output IN1;   // c300.R
output IN2;   // c320.R
output DSW1;  // c340.R
output DSW2;  // c360.R

//
wire mreq = ~n_MREQ & n_RFSH;
wire mem_rd = mreq & ~n_RD;
wire mem_wr = mreq &  n_RD;

// memory
assign ROM  = mreq & ~A[15];                     // 0000-7fff
assign TRAM = mreq & (A[15:12] == 4'ha);         // a000-afff
assign OBJ0 = mreq & (A[15:10] == 6'b101100);    // b000-b3ff
assign OBJ1 = mreq & (A[15:10] == 6'b101101);    // b400-b7ff

// write port
assign SOUND = mem_wr & (A[15:8]==8'hc0); // c0xx.W
assign WDT   = mem_wr & (A[15:8]==8'hc2); // c2xx.W

wire w_c3xx = mem_wr & (A[15:8]==8'hc3);
assign IRQEN = w_c3xx & (A[3:1]==3'b000); // c300.W
assign FLIP  = w_c3xx & (A[3:1]==3'b001); // c302.W
assign IRQTRG= w_c3xx & (A[3:1]==3'b010); // c304.W
assign COIN1 = w_c3xx & (A[3:1]==3'b101); // c30a.W
assign COIN2 = w_c3xx & (A[3:1]==3'b110); // c30c.W

// read port
wire r_c3xx = mem_rd & (A[15:8]==8'hc3);

assign SCAN = mem_rd & (A[15:8]==8'hc0); // c0xx.R
assign DSW2 = mem_rd & (A[15:8]==8'hc2); // c2xx.R
assign IN0  = r_c3xx & (A[7:5]==3'b000); // c30x.R
assign IN1  = r_c3xx & (A[7:5]==3'b001); // c32x.R
assign IN2  = r_c3xx & (A[7:5]==3'b010); // c34x.R
assign DSW1 = r_c3xx & (A[7:5]==3'b011); // c36x.R

endmodule

