/////////////////////////////////////////////////////////////////////////////
//
// TilePilot clock generator
//
// Version : 1.00
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// It isn't suitable to put this source on GPL. 
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source include this document.
//   2)Include all these headers , and add document of 
//     modification points
//
// Issue:
//
// Histry:
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

module tp_clk(CLKIN,PHASE3X,CLK_2_3,CLK_1_6,CLK_1_6_n);

input  CLKIN;       // 18.432MHz duty 50% base clock
output [2:0] PHASE3X; // clock enable 3turn raise of CLKIN
output CLK_2_3;     // 2/3 divide clock
output CLK_1_6;
output CLK_1_6_n;

//CLKIN    10101010101010101010
//CLK3P[0] 11000011000011000011
//CLK3P[1] 00110000110000110000
//CLK3P[2] 00001100001100001100
//c3n_1    00011000011000011000
//
//CLK2_3   11011011011011011011
//PCLK1_6  11100011100011100011
//NCLK1_6  00011100011100011100

// 3x raise clk enable
reg [2:0] c3p;
reg c3n_1;
always @(posedge CLKIN)
begin
  c3p[0]   <= (c3p[1:0]==2'b00);
  c3p[2:1] <= c3p[1:0];
end
// 3x fall clk enable
always @(negedge CLKIN) c3n_1 <= c3p[1];

assign PHASE3X = c3p;

// 2/3 clock duty 66%
assign CLK_2_3 = c3p[0] | c3n_1;

// 1/3 clock duty 50%
reg c16,c16n;
always @(posedge CLK_2_3)
begin
  c16n <= c3p[1];
  c16  <= ~c16;
end
assign CLK_1_6   = c16;
assign CLK_1_6_n = c16n;

endmodule
