/////////////////////////////////////////////////////////////////////////////
//
// TilePilot inport interface
//
// Version : 0.90
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// It isn't suitable to put this source on GPL. 
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source include this document.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
//   2004. 8.12 added FLIP controll
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

module tp_inp(
//
  CLK,
  vcnt,
  DI,
// inputs
  LT,RT,UP,DN,BT1,BT2,BT3,
  START,COIN,SERV,
  DSW1,DSW2,
//read enable
  SCAN_S,
  IN0_S,IN1_S,IN2_S,
  DSW1_S,DSW2_S,
//write pulse
  FLIP_W,
// output
  DO,
  FLIP
);

// System
input CLK;
input [7:0] vcnt;
input [7:0] DI;

// HID
input [1:0] LT,RT,UP,DN,BT1,BT2,BT3;
input [1:0] START,COIN;
input SERV;

// DSW
input [7:0] DSW1,DSW2;

// select
input SCAN_S;
input IN0_S,IN1_S,IN2_S;
input DSW1_S,DSW2_S;

// latch
input FLIP_W;

// Output
output [7:0] DO;
output reg FLIP;

/////////////////////////////////////////////////////////////////////////////
// controller & dip sw.
/////////////////////////////////////////////////////////////////////////////
wire [7:0] in0do  = {8{IN0_S}} & { 3'b111,START[1],START[0],SERV,COIN[1],COIN[0] };
wire [7:0] in1do  = {8{IN1_S}} & { 1'b1,BT3[0],BT2[0],BT1[0],DN[0],UP[0],RT[0],LT[0]};
wire [7:0] in2do  = {8{IN2_S}} & { 1'b1,BT3[1],BT2[1],BT1[1],DN[1],UP[1],RT[1],LT[1]};
wire [7:0] dsw1do = {8{DSW1_S}} & DSW1;
wire [7:0] dsw2do = {8{DSW2_S}} & DSW2;

/////////////////////////////////////////////////////////////////////////////
// scanline
/////////////////////////////////////////////////////////////////////////////
wire [7:0] scan_do = {8{SCAN_S}} & vcnt;

/////////////////////////////////////////////////////////////////////////////
// output bus
/////////////////////////////////////////////////////////////////////////////
reg [7:0] do_r;
always @(posedge CLK)
begin
  do_r <= in0do | in1do | in2do | dsw1do | dsw2do | scan_do;
end
assign DO = do_r;

/////////////////////////////////////////////////////////////////////////////
// flip controll
/////////////////////////////////////////////////////////////////////////////
always @(negedge FLIP_W)
begin
  FLIP <= ~DI[0];
end

endmodule
