////////////////////////////////////////////////////////////////////////////
//
// FPGA TimePilot Main Circuit
//
// Version : 1.00
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL,
// It isn't suitable to put this source on GPL. 
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source include this document.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
//   2004. 8.12 Support flip screen & Player2 Controll
//   2004. 8.10 Bug Fix on Altera Quartus2 v4.0sp1 (by K.Degawa)
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

// ----- option switch -----
`include "src/switch.vh"

// ---- default set -----

//`define SPARTAN2_IO // Xilinx SpartanII , SpartanIIE
//`define CYCLONE_IO // Altera Cyclone

`ifndef MCPU_MODULE
  `define MCPU_MODULE T80as     // T80 full spec
`endif

module tp_main(
// 18.432MHz generic CLOCK input
  RST_N,
  CLK18M432,
// Video I/F
  pclk,dclk,R,G,B,hsync,vsync,
// Controller
  UP,DN,LT,RT,B1,B2,B3,CO,ST,SV,
// Dip Switchs
  DSW1,DSW2,
//Sound
  M_DO,SCMD_W,SIRQ_W,

// Additional Timing Signal
  PHASE3x,HCNT,

// Main CPU ROM
  M_A,MROM_DI,MROM_OE,
// Tilemap Pattern ROM
  TROM_A,TROM_DI,
// Object Pattern ROM
  OROM_A,OROM_DI,
// MainCPU Write Data
  M_WD,
// TileMap RAM
  TRAM_A,TRAM_DI,TRAM_CS,TRAM_WE,
// TileMap ColorLookUpTable ROM
  TCLUT_A,TCLUT_DI,
// Object RAM
  OBJ_A,OBJ0_DI,OBJ0_WE,OBJ1_DI,OBJ1_WE,
// Object Line RAM
  L0A,L0DI,L0DO,L0WE,
  L1A,L1DI,L1DO,L1WE,
// OBJ ColorLookUpTable ROM
  OCLUT_A,OCLUT_DI,
// Palette ROM
  PAL_A,PAL_EN,PAL_DI
);

/////////////////////////////////////////////////////////////////////////////
// module I/O assign
/////////////////////////////////////////////////////////////////////////////

// 18.432MHz generic CLOCK input
input RST_N;
input CLK18M432; // 18.432MHz

//Sound
//output [9:0] PCM;

// Universal Video I/F
output pclk,dclk;
output [4:0] R,G,B;
output hsync,vsync;

// Controller
input [1:0] UP,DN,LT,RT,B1,B2,B3,CO,ST;
input SV;

// Dip Switch
input [7:0] DSW1,DSW2;

// Sound
output [7:0] M_DO;
output SCMD_W,SIRQ_W;

//
output [2:0] PHASE3x;
output [8:0] HCNT;

// Main CPU ROM
output [14:0] M_A;
input [7:0] MROM_DI;
output MROM_OE;

// Tilemap Pattern ROM
output [12:0] TROM_A;
input [7:0] TROM_DI;
// Object Pattern ROM
output [13:0] OROM_A;
input [7:0] OROM_DI;

// TileMap RAM & OBJect RAM Write data
output [7:0] M_WD;

// TileMap RAM
output [11:0] TRAM_A;
input [7:0] TRAM_DI;
output TRAM_CS,TRAM_WE;

// TileMap ColorLookUpTable ROM
output [7:0] TCLUT_A;
input [3:0] TCLUT_DI;

// Object RAM
output [7:0] OBJ_A;
input [7:0] OBJ0_DI,OBJ1_DI;
output OBJ0_WE,OBJ1_WE;

//Object ColorLookUpTable ROM
output [7:0] OCLUT_A;
input [3:0] OCLUT_DI;

// Object Line RAM
output [7:0] L0A,L1A;
output [3:0] L0DO,L1DO;
input  [3:0] L0DI,L1DI;
output L0WE,L1WE;

// Palette ROM
output [4:0] PAL_A;
output PAL_EN;
input [14:0] PAL_DI;

/////////////////////////////////////////////////////////////////////////////
// clock generator
/////////////////////////////////////////////////////////////////////////////
wire CLK18_432M = CLK18M432;
wire CLK12_288M;
wire CLK6_144M;
wire [2:0] phase_3p;
tp_clk tp_clk(
	.CLKIN(CLK18_432M),
	.PHASE3X(phase_3p),
	.CLK_2_3(CLK12_288M),
	.CLK_1_6(CLK6_144M),
	.CLK_1_6_n()
);

wire pixel_clk;

// Grobal Clock to video pixel clock
`ifdef SPARTAN2_IO
  // Xilinx SpartanII
  BUFG PIXEL_CLOCK(.I(CLK6_144M),.O(pixel_clk));
`elsif CYCLONE_IO
  // altera cyclone
  assign pixel_clk = CLK6_144M;
`else
  // standard syntax
  assign pixel_clk = CLK6_144M;
`endif

/////////////////////////////////////////////////////////////////////////////
// Main CPU
/////////////////////////////////////////////////////////////////////////////

wire [15:0] ZA;
wire [7:0] ZDO,ZDI;
wire ZMREQ,ZRD,ZWR,ZRFSH;
wire Z80CLK,ZNMI,ZRESET,ZWAIT;

`MCPU_MODULE Main_CPU(
  .CLK_n(Z80CLK),
  .RESET_n(ZRESET),
  .A(ZA), .DO(ZDO), .DI(ZDI), .DOE(),
  .M1_n(), .MREQ_n(ZMREQ), .IORQ_n(), 
  .RD_n(ZRD), .WR_n(ZWR),
  .WAIT_n(ZWAIT), .RFSH_n(ZRFSH),
  .INT_n(1'b1), .NMI_n(ZNMI), .HALT_n(),
  .BUSRQ_n(1'b1),.BUSAK_n()
);

/////////////////////////////////////////////////////////////////////////////
// address decoder
/////////////////////////////////////////////////////////////////////////////

wire tram_as;
wire obj0_as,obj1_as;
wire irqen_wr;
wire flip_wr , flip;
wire snd_irq_wr;
wire scan_r;
wire in0_r,in1_r,in2_r,dsw1_r,dsw2_r;

wire snd_cmd_wr;

tp_adec timeplt_adec(
// inputs
	.A(ZA),
	.n_MREQ(ZMREQ),
	.D0I(ZDO[0]),
	.n_RD(ZRD),
	.n_RFSH(ZRFSH),
// decode output
	.ROM(MROM_OE),
	.TRAM(tram_as),
	.OBJ0(obj0_as),
	.OBJ1(obj1_as),
//
  .SOUND(SCMD_W),
  .WDT(),
  .IRQEN(irqen_wr), // 2004.8.10 Bug Fix
  .FLIP(flip_wr),   // 2004.8.10 Bug Fix
  .IRQTRG(SIRQ_W),
  .COIN1(),
  .COIN2(),
//
  .SCAN(scan_r),
  .DSW2(dsw2_r),
  .IN0(in0_r),
  .IN1(in1_r),
  .IN2(in2_r),
  .DSW1(dsw1_r)
);

/////////////////////////////////////////////////////////////////////////////
// VIDEO hardware interface
/////////////////////////////////////////////////////////////////////////////

wire vwait_n;
wire [8:0] hcnt;
wire [7:0] vcnt;
wire hsync,vsync;
wire hwindow,vwindow;
wire [7:0] vid_do;
wire [4:0] red;
wire [4:0] green;
wire [4:0] blue;

///////////////////////////////////////////////////////////
// VIDEO Timming generator
///////////////////////////////////////////////////////////
tp_vtmg video_timming(
  .rst_n(ZRESET),
  .pclk(pixel_clk),
  .hcnt(hcnt),  .hwindow(hwindow), .hsync(hsync),
  .vcnt(vcnt),  .vwindow(vwindow), .vsync(vsync)
 );

///////////////////////////////////////////////////////////
// video module
///////////////////////////////////////////////////////////
tp_vid timeplt_video(
// CPU bus
  .A(ZA[11:0]), .di(ZDO), .do(vid_do),
  .n_rd(ZRD), .wait_n(vwait_n),
// Address Decode
  .tram_as(tram_as),.oram0_as(obj0_as),.oram1_as(obj1_as),
// Scan Timming Input
  .pclk(pixel_clk),
  .pclk2x(CLK12_288M),
  .flip(flip),
  .hcnt(hcnt),     .vcnt(vcnt),
  .hwindow(hwindow), .vwindow(vwindow),
// Tilemap RAM
  .TRAM_A(TRAM_A),.TRAM_DI(TRAM_DI),.TRAM_WE(TRAM_WE), .TRAM_CS(TRAM_CS),
// Tilemap Pattern ROM
  .TPAT_A(TROM_A),  .TPAT_DI(TROM_DI),
// Tilemap ColorLookupTable ROM
  .TCLUT_A(TCLUT_A), .TCLUT_DI(TCLUT_DI),
// Object RAM
  .ORAM_A(OBJ_A), .ORAM0_DI(OBJ0_DI), .ORAM1_DI(OBJ1_DI),
  .ORAM0_WE(OBJ0_WE), .ORAM1_WE(OBJ1_WE),
// OBJ pattern ROM
  .OPAT_A(OROM_A),.OPAT_DI(OROM_DI),
// Object ColorLookupTable ROM
  .OCLUT_A(OCLUT_A), .OCLUT_DI(OCLUT_DI),
// Object LINE RAM
  .LRAM0_A(L0A),.LRAM0_DO(L0DO),.LRAM0_DI(L0DI),.LRAM0_WE(L0WE),
  .LRAM1_A(L1A),.LRAM1_DO(L1DO),.LRAM1_DI(L1DI),.LRAM1_WE(L1WE),
// PaletteROM
 .PAL_A(PAL_A), .PAL_EN(PAL_EN)
);

/////////////////////////////////////////////////////////////////////////////
// R,G,B output
/////////////////////////////////////////////////////////////////////////////
assign B = PAL_DI[14:10];
assign G = PAL_DI[ 9: 5];
assign R = PAL_DI[ 4: 0];

/////////////////////////////////////////////////////////////////////////////
// IRQ controller
/////////////////////////////////////////////////////////////////////////////
tp_irq tileplt_irq(
  .vwindow(vwindow),
  .wr(irqen_wr),  // 2004.8.10 Bug Fix
  .di(ZDO),
  .irq_n(ZNMI)
);

/////////////////////////////////////////////////////////////////////////////
// controller IN / DIP switch I/F
/////////////////////////////////////////////////////////////////////////////
//                  Player2        , Player1
wire [7:0] inp_do;

tp_inp timeplt_inport(
// system
  .CLK(pixel_clk),
  .vcnt(vcnt),
  .DI(ZDO),
// inputs
  .LT(LT), .RT(RT), .UP(UP), .DN(DN), .BT1(B1),.BT2(B2),.BT3(B3),
  .START(ST), .COIN(CO),
  .SERV(SV),
  .DSW1(DSW1),.DSW2(DSW2),
//read port
  .SCAN_S(scan_r),
  .IN0_S(in0_r),.IN1_S(in1_r),.IN2_S(in2_r),
  .DSW1_S(dsw1_r),.DSW2_S(dsw2_r),
//write port
  .FLIP_W(flip_wr),
//output
  .DO(inp_do),
  .FLIP(flip)
);

/////////////////////////////////////////////////////////////////////////////
// CPU clock assign
/////////////////////////////////////////////////////////////////////////////

// Grobal Clock to CPU
`ifdef SPARTAN2_IO
  // Xilinx SpartanII
  BUFG ZCLK(.I(~hcnt[0]),.O(Z80CLK));
`elsif CYCLONE_IO
  // altera cyclone
  assign Z80CLK = ~hcnt[0];
`else
  // standard syntax
  assign Z80CLK = ~hcnt[0];
`endif

/////////////////////////////////////////////////////////////////////////////
// Main CPU data bus multiplexer
/////////////////////////////////////////////////////////////////////////////

assign ZDI   = (vid_do | MROM_DI | inp_do);
assign ZWAIT = vwait_n;

assign ZRESET = RST_N;

/////////////////////////////////////////////////////////////////////////////
// output assign
/////////////////////////////////////////////////////////////////////////////
assign pclk = pixel_clk;
assign dclk = CLK12_288M;

// Snd
assign M_DO = ZDO;

assign PHASE3x = phase_3p;
assign HCNT = hcnt;

//MainCPU ROM
assign M_A = ZA[14:0];

// TileMap RAM & OBJect RAM Write data
assign M_WD = ZDO;

endmodule
