/////////////////////////////////////////////////////////////////////////////
//
// TimePilot sound filter & mixer
//
// Version : 0.90
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// It isn't suitable to put this source on GPL. 
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source include this document.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
//   2004. 7.23 1st Release
//
/////////////////////////////////////////////////////////////////////////////
module mixer(CLK,A,filter_w,A0,B0,C0,A1,B1,C1,O);

input CLK;            // filter clock
input [11:0] A;       // CPU Address Bus
input filter_w;       // filter Write Access
input [7:0] A0,B0,C0; // sound in
input [7:0] A1,B1,C1; // sound in
output [9:0] O;

/////////////////////////////////////////////////////////////////////////////
// CR filter select
/////////////////////////////////////////////////////////////////////////////
reg [1:0] fsel_a0,fsel_b0,fsel_c0;
reg [1:0] fsel_a1,fsel_b1,fsel_c1;

always @(posedge filter_w)
begin
  fsel_a0 <= A[ 7: 6];
  fsel_b0 <= A[ 9: 8];
  fsel_c0 <= A[11:10];
  fsel_a1 <= A[ 1: 0];
  fsel_b1 <= A[ 3: 2];
  fsel_c1 <= A[ 5: 4];
end

/////////////////////////////////////////////////////////////////////////////
// CR filter module
/////////////////////////////////////////////////////////////////////////////
wire [7:0] fa0,fb0,fc0;
wire [7:0] fa1,fb1,fc1;

rc_filt filter_a0(.CLK(CLK),.SEL(fsel_a0),.I(A0),.O(fa0));
rc_filt filter_b0(.CLK(CLK),.SEL(fsel_b0),.I(B0),.O(fb0));
rc_filt filter_c0(.CLK(CLK),.SEL(fsel_c0),.I(C0),.O(fc0));
rc_filt filter_a1(.CLK(CLK),.SEL(fsel_a1),.I(A1),.O(fa1));
rc_filt filter_b1(.CLK(CLK),.SEL(fsel_b1),.I(B1),.O(fb1));
rc_filt filter_c1(.CLK(CLK),.SEL(fsel_c1),.I(C1),.O(fc1));

/////////////////////////////////////////////////////////////////////////////
// mixing out
/////////////////////////////////////////////////////////////////////////////
wire [10:0] mix = fa0 + fb0 + fc0 + fa1 + fb1 + fc1;
assign O = mix[9:0]; // clip MSB

endmodule
