/////////////////////////////////////////////////////////////////////////////
//
// FPGA TimePilot SubBoard
//
// Version : 0.90
//
// Copyright(c) 2004 Tatsuyuki Satoh , All rights reserved
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// It isn't suitable to put this source on GPL. 
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source include this document.
//   2)Include all these headers , and add document of 
//     modification points
//
// Histry:
//   2004. 7.21 1st Release
//
/////////////////////////////////////////////////////////////////////////////

// ----- option switch -----
`include "src/switch.vh"

// ---- default set -----
`ifndef SCPU_MODULE
  `define SCPU_MODULE T80as    // T80 full spec
`endif

module tp_sub(
  rst_n,
  clk17m8, // 18.432M * 3 / 31
// I/O I/F
  snd_di,
  snd_cmd_w,snd_irq_w,
// ROM/RAM
  mem_a,
  rom_di, rom_oe,
  ram_cs, ram_we, ram_di, ram_do,
// sound I/F
  snd_out
);

input rst_n;
input clk17m8;

input [7:0] snd_di;
input snd_cmd_w;
input snd_irq_w;

// ROM/RAM addreee
output [12:0] mem_a;
// ROM
output rom_oe;
input [7:0] rom_di;
// RAM
output ram_cs;
output ram_we;
output [7:0] ram_do;
input [7:0] ram_di;

output [9:0] snd_out;

/////////////////////////////////////////////////////////////////////////////
// Z80 SUB CPU
/////////////////////////////////////////////////////////////////////////////
wire [15:0] ZA;
wire [7:0] ZDO,ZDI;
wire ZRESET,ZMREQ,ZM1,ZIORQ,ZRD,ZWR,ZRFSH;
wire ZINT;

// Z80IP interface
`SCPU_MODULE z80_sub (
	.RESET_n(rst_n),
	.CLK_n(clk17m8),
	.WAIT_n(1'b1),
	.INT_n(ZINT),
	.NMI_n(1'b1),
	.BUSRQ_n(1'b1),
	.M1_n(ZM1),
	.MREQ_n(ZMREQ),
	.IORQ_n(ZIORQ),
	.RD_n(ZRD),
	.WR_n(ZWR),
	.RFSH_n(ZRFSH),
	.HALT_n(),
	.BUSAK_n(),
	.A(ZA),
	.DI(ZDI),
	.DO(ZDO),
	.DOE()
);

/////////////////////////////////////////////////////////////////////////////
// Address Decoder
/////////////////////////////////////////////////////////////////////////////

wire [7:0] rom_rd = rom_di;
wire [7:0] ram_rd = ram_di;
wire [7:0] ay0_rd,ay1_rd;

wire m_req   = ~ZMREQ;// & ZRFSH;
wire rom_ce  = m_req & (ZA[15:13]==3'b000); // 0000-1fff
wire ram_ce  = m_req & (ZA[15:12]==4'h3);   // 3000-3fff
wire ay0ce   = m_req & (ZA[15:13]==3'h2);   // 4000-5fff
wire ay1ce   = m_req & (ZA[15:13]==3'h3);   // 6000-7fff
wire filt_w  = ~ZWR & ~ZA[15];              // 8000-ffff.w

assign ZDI = rom_rd | ram_rd | ay0_rd | ay1_rd;

/////////////////////////////////////////////////////////////////////////////
// IRQ signale
/////////////////////////////////////////////////////////////////////////////
wire irq_clr = (~ZM1 & ~ZIORQ) | ~rst_n;
reg irq_n_r;

always @(negedge snd_irq_w or posedge irq_clr)
begin
  if(irq_clr) irq_n_r = 1'b1;
  else        irq_n_r = 1'b0;
end
assign ZINT = irq_n_r;

/////////////////////////////////////////////////////////////////////////////
// sound command
/////////////////////////////////////////////////////////////////////////////
reg [7:0] snd_cmd_r;
always @(negedge snd_cmd_w or negedge rst_n)
begin
  if(~rst_n)
    snd_cmd_r <= 8'h00;
  else
    snd_cmd_r <= snd_di;
end

/////////////////////////////////////////////////////////////////////////////
// timming counter
/////////////////////////////////////////////////////////////////////////////
reg [8:0] pris;
always @(posedge clk17m8)
begin
  pris <= pris + 1;
end

wire tick_c = pris[8];

reg [3:0] cnt10;
always @(negedge tick_c)
begin
  if(cnt10[2])
	begin
	  cnt10[2:0] <= 0;
		cnt10[3]   <= ~cnt10[3];
	end else begin
		cnt10[2:0] <= cnt10[2:0]+1;
  end
end

// QA,QD,QC of 74ls90 and clkb of 74ls90
wire [7:0] cnt_r = {cnt10[3:1],tick_c,4'b0000};

/////////////////////////////////////////////////////////////////////////////
// AY-3-8910 PSG x2
/////////////////////////////////////////////////////////////////////////////
wire [7:0] pcm0a,pcm0b,pcm0c,pcm1a,pcm1b,pcm1c;

ay8910 ay8910_0(
  .rst_n(rst_n), .clk(clk17m8),
  .asel(ZA[12]), .cs_n(~ay0ce), .wr_n(ZWR), .rd_n(ZRD),
  .di(ZDO), .do(ay0_rd),
  .A(pcm0a),.B(pcm0b),.C(pcm0c),
  .pa_i(snd_cmd_r),.pa_o(),.pa_t(),
  .pb_i(cnt_r),.pb_o(),.pb_t()
);

ay8910 ay8910_1(
  .rst_n(rst_n), .clk(clk17m8),
  .asel(ZA[12]), .cs_n(~ay1ce), .wr_n(ZWR), .rd_n(ZRD),
  .di(ZDO), .do(ay1_rd),
  .A(pcm1a),.B(pcm1b),.C(pcm1c),
  .pa_i(8'h00),.pa_o(),.pa_t(),
  .pb_i(8'h00),.pb_o(),.pb_t()
);

assign mem_a  = ZA[12:0];
assign rom_oe = rom_ce;
assign ram_cs = ram_ce;
assign ram_we = ram_ce & ~ZWR;
assign ram_do = ZDO;

/////////////////////////////////////////////////////////////////////////////
// Sound Filter & Mixer
/////////////////////////////////////////////////////////////////////////////
mixer mixer(
  .CLK(clk17m8),.A(ZA[11:0]),.filter_w(filt_w),
  .A0(pcm0a),.B0(pcm0b),.C0(pcm0c),
  .A1(pcm1a),.B1(pcm1b),.C1(pcm1c),
  .O(snd_out)
);

endmodule

